#!/usr/bin/env python
#
# Copyright 2009 Canonical, Ltd
# Author: Brian Murray <brian@ubuntu.com>
# Author: Mathias Gug <mathias.gug@canonical.com>
# License: GPLv2
# 
# 2009-07-31
# This will create a report of all the New, Undecided bugs 
# with a date_created < end_date for packages to which a team
# is a bug supervisor

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import datetime, os, os.path, re, shutil, sys, urllib

team_name = "ubuntu-server"
end_date = datetime.date.today() - datetime.timedelta(1)
cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))
    
def save_entry(entry):
    try:
        entry.lp_save()
    except HTTPError, error:
        print error.content
    else:
        print "Did not get expected HTTPError!"

# write to this file
data_filename = "%s.%s.%s.html" % (script_name, team_name, end_date)
datafile = open(data_filename,'w')

# commonly used things
ubuntu = launchpad.distributions['ubuntu']
team = launchpad.people[team_name]
templatefile = open('template.html.tmpl', 'r')

for line in templatefile:
    if line.strip() == "<!-- *** Title Space *** -->":
        datafile.write("\n")
    elif line.strip() == "<!-- *** Header Space *** -->":
        datafile.write("%s New, Unconfirmed bugs on %s\n"\
                       % (team_name, end_date))
    elif line.strip() == "<!-- *** Paragraph Space *** -->":
        datafile.write("All of the column headers are sortable give them a click!")
    elif line.strip() == "<!-- *** Table Header Space *** -->":
        table_header = "<th></th>"  # bug number in this column
        table_header += "<th>Summary</th>"
        table_header += "<th>Package</th>"
        table_header += "<th>Date Created</th>"
        table_header += "\n"
        datafile.write(table_header)
    elif line.strip() == "<!-- *** Table Body Space *** -->":
        for task in ubuntu.searchTasks(order_by='-datecreated',
                                       bug_supervisor=team.self_link,
                                       status=['New'],
                                       importance=['Undecided']):
            if task.date_created.date() <= end_date:
                try:
                    table_row = "<tr>"
                    table_row += "<td>%s</td>" % task.bug.id
                    table_row += "<td><a href='http://launchpad.net/bugs/%s'>%s</a></td>" % (task.bug.id, task.bug.title.encode('utf-8'))
                    table_row += "<td>%s</td>" % task.bug_target_display_name.encode('utf-8')
                    table_row += "<td>%s</td>" % task.date_created.strftime('%Y-%m-%d')
                    table_row += "</tr>"
                    table_row += "\n"
                    datafile.write(table_row)
                except HTTPError, error:
                    print "There was an error with LP: #%s: %s" % (task.title.split(' ')[1].replace('#',''), error.content)
    else:
        datafile.write(line)
datafile.close()

weekday = end_date.strftime("%a").lower()
try:
    os.unlink("%s.%s.%s.html" % (script_name, team_name, weekday))
except OSError:
    pass
shutil.copy(data_filename, "%s.%s.%s.html" % (script_name, team_name, weekday))

