#!/usr/bin/env python
#
# Copyright 2009 Canonical, Ltd
# Author: Brian Murray <brian@ubuntu.com>
# Author: Mathias Gug <mathias.gug@canonical.com>
# License: GPLv2
# 
# 2009-07-31
# This will create a report of all the Fix Released bugs 
# with a date_fix_released > start_date for a list of packages
# provided by a text file (packagefile)

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import datetime, os, re, sys, urllib

# pattern used to extract the list of packages subscribed to by the team
p = re.compile('>([^\s]+)\sin\subuntu')

team_name = "ubuntu-server"
end_date = datetime.datetime.now().strftime('%Y-%m-%d')
cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))
    
packages = [ ]

(f_name, h) = urllib.urlretrieve(\
                    "https://bugs.launchpad.net/~%s/+packagebugs" %
                    team_name)
for l in open(f_name):
    m = p.search(l)
    if m:
        packages.append(m.group(1))

target_packages = [ ]

# write to this file
datafile = open("nominated-bugs.%s.%s.html"\
                % (team_name, end_date), 'w')

# commonly used things
ubuntu = launchpad.distributions['ubuntu']

for package in packages:
    target_packages.append(ubuntu.getSourcePackage(name='%s' % package))

#tasks = target_package.searchTasks(status=['New','Incomplete','Confirmed'],order_by='-datecreated')

templatefile = open('template.html', 'r')

for line in templatefile:
    if line.strip() == "<!-- *** Title Space *** -->":
        datafile.write("\n")
    elif line.strip() == "<!-- *** Header Space *** -->":
        datafile.write("%s nominated bugs - %s\n"\
                       % (team_name, end_date))
    elif line.strip() == "<!-- *** Paragraph Space *** -->":
        datafile.write("All of the column headers are sortable give them a click!")
    elif line.strip() == "<!-- *** Table Header Space *** -->":
        table_header = "<th></th>"  # bug number in this column
        table_header += "<th>Summary</th>"
        table_header += "<th>In</th>"
        table_header += "<th>Importance</th>"
        table_header += "<th>Nominated for</th>"
        table_header += "<th>Date Fixed</th>"
        table_header += "\n"
        datafile.write(table_header)
    elif line.strip() == "<!-- *** Table Body Space *** -->":
        for distro in 'dapper', 'hardy', 'intrepid', 'jaunty':
            distro_serie = ubuntu.getSeries(name_or_version=distro)
            for target_package in target_packages:
                for task in target_package.searchTasks(\
                                           status=['Fix Released'],
                                           omit_targeted=False,
                                           nominated_for=distro_serie):
                    try:
                            table_row = "<tr>"
                            table_row += "<td>%s</td>" % task.bug.id
                            table_row += "<td><a href='http://launchpad.net/bugs/%s'>%s</a></td>" % (task.bug.id, task.bug.title.encode('utf-8'))
                            table_row += "<td>%s</td>" % task.bug_target_display_name.encode('utf-8')
                            table_row += "<td>%s</td>" % task.importance.encode('utf-8')
                            table_row += "<td>%s</td>" % distro
                            if task.date_fix_released:
                                table_row += "<td>%s</td>" % task.date_fix_released.strftime('%Y-%m-%d')
                            elif task.date_closed:
                                table_row += "<td>%s</td>" % task.date_closed.strftime('%Y-%m-%d')
                            else:
                                table_row += "<td>Unknown</td>"
                            table_row += "</tr>"
                            table_row += "\n"
                            datafile.write(table_row)
                    except HTTPError, error:
                            print "There was an error with LP: #%s: %s" % (task.title.split(' ')[1].replace('#',''), error.content)
    else:
        datafile.write(line)
datafile.close()
